require 'faraday'
require 'faraday_middleware'
require_relative 'request'
require_relative 'requests'

module Laa
  module ApplyForLegalAid
    module Api
      class Client
        attr_reader :host

        include Requests

        def initialize(host, options = {})
          @host = host
          @options = options
        end

        def method_missing(method_name, *args, &block)
          return super unless Request::HTTP_METHODS.include?(method_name)
          path = args.shift
          req_options = args.extract_options!
          Request.new(method_name, path, connection).call(req_options)
        end

        private

        def connection
          @connection ||= initialize_connection
        end

        def initialize_connection
          Faraday.new(url: host) do |conn|
            conn.request :json
            conn.response :json, content_type: /\bjson$/
            conn.adapter Faraday.default_adapter
          end
        end
      end
    end
  end
end
