module Laa
  module ApplyForLegalAid
    module Api
      class Request
        HTTP_METHODS = [:get, :head, :post, :put, :delete, :patch].freeze
        HTTP_METHODS_WITH_BODY = [:post, :put, :patch].freeze

        attr_reader :action, :path

        def initialize(action, path, connection)
          @action = action
          @path = path
          @connection = connection
          validate_http_method
        end

        def call(options = {})
          # TODO: wrap the response in a response object we own
          connection.send(action) do |request|
            request.url(path, options[:query] || {})
            request.body = request_body(options[:data]) if body_required? && options.has_key?(:data)
          end
        end

        private

        attr_reader :connection

        def validate_http_method
          return if HTTP_METHODS.include?(action)
          raise ArgumentError, "invalid HTTP method #{action}"
        end

        def body_required?
          HTTP_METHODS_WITH_BODY.include?(action)
        end

        def request_body(data)
          return data if data.is_a?(String)
          data.to_json
        end
      end
    end
  end
end
